
subroutine CalcEnergy(Pos, M, L, d, rc, PEnergy, Dim, NAtom)
    implicit none
    integer, intent(in) :: M, Dim, NAtom
    real(8), intent(in), dimension(0:NAtom-1, 0:Dim-1) :: Pos
    real(8), intent(in) :: L, rc, d
    real(8), intent(out) :: PEnergy
    real(8), parameter :: k = 3000., r0 = 1., Pi = 3.1415927, pre=0.98666667
    !pre is a prefactor for the LJ 9-3 wall potential
    real(8), dimension(Dim) :: rij, Posi
    real(8) :: d2, id2, id6, id12, rc2, Shiftun, Shiftwall, Poszid
    integer::i,j
    rc2 = rc * rc
    Shiftun = -4. * (rc**(-12) - rc**(-6))
    Shiftwall =  -pre * Pi * ((6./15.)**(-9./6.)/15. - (6./15.)**(-3./6.)/2.) !cutoff at rc=(6/15)**(1/6)
    PEnergy = 0.
    do i = 0 , NAtom - 1
        Posi = Pos(i,:)
	if (mod(i,M)>0) then
	    !repulsive wall interaction using a LJ 9-3 potential with cut and shift
            if (abs(Posi(3)) < abs(Posi(3)-d)) then
                !lower wall
                Poszid = 1/Posi(3)
		PEnergy = PEnergy + (pre * Pi * ((Poszid)**9 / 15.))
            else
                !upper wall
                Poszid = 1/(d-Posi(3))
		PEnergy = PEnergy + (pre * Pi * ((Poszid)**9 / 15.))
            endif
	endif
        do j = i + 1 , NAtom - 1
            rij=Pos(j,:)-Posi
            !compute min image distance for x and y directions only
            rij(1)=rij(1) - L * dnint(rij(1) / L)
            rij(2)=rij(2) - L * dnint(rij(2) / L)
            d2 = sum(rij * rij)
            if (j==i+1 .and. mod(j,M)>0) then
                !bonded interaction
                PEnergy = PEnergy + ((k/2.) * (sqrt(d2)-r0)**2.)
            else if (d2 < rc2) then
                !unbonded interaction
                id2 = 1./d2
                id6 = id2 * id2 * id2
                id12 = id6 * id6
                PEnergy = PEnergy + (4. * (id12-id6)) + Shiftun
            endif
        enddo
    enddo

end subroutine


subroutine CalcEnergyForces(Pos, M, L, d, rc, PEnergy, Forces, Dim, NAtom)
    implicit none
    integer, intent(in) :: M, Dim, NAtom
    real(8), intent(in), dimension(0:NAtom-1, 0:Dim-1) :: Pos
    real(8), intent(in) :: L, rc, d
    real(8), intent(out) :: PEnergy
    real(8), intent(inout), dimension(0:NAtom-1, 0:Dim-1) :: Forces
    !f2py intent(in, out) :: Forces
    real(8), parameter :: k = 3000., r0 = 1., Pi = 3.1415927, pre=0.98666667
    !pre is a prefactor for the LJ 9-3 wall potential
    real(8), dimension(Dim) :: rij, Posi, Fij
    real(8) :: d2, id2, id6, id12, rc2, Shiftun, Shiftwall, Poszid, Fwall
    integer::i,j, a
    rc2 = rc * rc
    Shiftun = -4. * (rc**(-12) - rc**(-6))
    Shiftwall =  -pre * Pi * ((6./15.)**(-9./6.)/15. - (6./15.)**(-3./6.)/2.)
    PEnergy = 0.
    Forces = 0.
    do i = 0 , NAtom - 1
	Posi = Pos(i,:)
	if (mod(i,M)>0) then
	    !repulsive wall interaction using a LJ 9-3 potential with cut and shift
                if (abs(Posi(3)) < abs(Posi(3)-d)) then
                    !lower wall
                    Poszid = 1/Posi(3)
                    PEnergy = PEnergy + (pre * Pi * ((Poszid)**9 / 15.))
                    Fwall = pre * Pi * (9. * (Poszid)**10 / 15. - 3.)
                else
                    !upper wall
                    Poszid = 1/(d-Posi(3))
                    PEnergy = PEnergy + (pre * Pi * ((Poszid)**9 / 15.))
                    Fwall = pre * Pi * (-9. * (Poszid)**10 / 15. + 3.)
                endif
	    Forces(i,2) = Forces(i,2) + Fwall
	endif
        do j = i + 1 , NAtom - 1
	    rij=Pos(j,:)-Posi
	    rij(1)=rij(1) - L * dnint(rij(1) / L)
	    rij(2)=rij(2) - L * dnint(rij(2) / L)
	    d2 = sum(rij * rij)
	    if (j==i+1 .and. mod(j,M)>0) then
		!bonded interaction
		PEnergy = PEnergy + ((k/2.) * (sqrt(d2)-r0)**2.)
		Fij = rij * k * (1-(r0/sqrt(d2)))
		Forces(i,:) = Forces(i,:) + Fij
		Forces(j,:) = Forces(j,:) - Fij
	    else if (d2 < rc2) then
		!unbonded interaction
		id2 = 1./d2
		id6 = id2 * id2 * id2
		id12 = id6 * id6
		PEnergy = PEnergy + (4. * (id12-id6)) + Shiftun
		Fij = rij * ((-48. * id12 + 24. * id6) * id2)
		Forces(i,:) = Forces(i,:) + Fij
		Forces(j,:) = Forces(j,:) - Fij
	    endif
        enddo
    enddo
end subroutine


subroutine VVIntegrate(Pos, Vel, Accel, M, L, d, rc, dt, KEnergy, PEnergy, Dim, NAtom)
    implicit none
    integer, intent(in) :: M, Dim, NAtom
    real(8), intent(in) :: L, rc, dt, d
    real(8), intent(inout), dimension(0:NAtom-1, 0:Dim-1) :: Pos, Vel, Accel
!f2py intent(in,out) :: Pos, Vel, Accel
    real(8), intent(out) :: KEnergy, PEnergy
    external :: CalcEnergyForces
    Pos = Pos + dt * Vel + 0.5 * dt*dt * Accel 
    Vel = Vel + 0.5 * dt * Accel
    call CalcEnergyForces(Pos, M, L, d, rc, PEnergy, Accel, Dim, NAtom)
    Vel = Vel + 0.5 * dt * Accel
    KEnergy = 0.5 * sum(Vel*Vel)
end subroutine